package dev.pfaff.altargui.pathfinder;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;

import static dev.pfaff.altargui.pathfinder.Pathfinder.*;

public record PFSceneProxyRef(MemorySegment ptr) {
	public static PFSceneProxyRef createWithSequentialExecutor(PFRendererMode mode) {
		try {
			MemorySegment ptr = (MemorySegment) MH_PFSceneProxyCreateWithSequentialExecutor.invokeExact(mode.value);
			return new PFSceneProxyRef(ptr.reinterpret(Arena.ofAuto(), ptr_ -> {
				try {
					MH_PFSceneProxyDestroy.invokeExact(ptr_);
				} catch (Throwable e) {
					throw new RuntimeException(e);
				}
			}));
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public static PFSceneProxyRef createWithRayonExecutor(PFRendererMode mode) {
		try {
			MemorySegment ptr = (MemorySegment) MH_PFSceneProxyCreateWithRayonExecutor.invokeExact(mode.value);
			return new PFSceneProxyRef(ptr.reinterpret(Arena.ofAuto(), ptr_ -> {
				try {
					MH_PFSceneProxyDestroy.invokeExact(ptr_);
				} catch (Throwable e) {
					throw new RuntimeException(e);
				}
			}));
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Ownership of the {@code scene} will be transferred by this call.
	 */
	public void replaceScene(PFSceneRef scene) {
		try {
			MH_PFSceneProxyReplaceScene.invokeExact(this.ptr, scene.ptr());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void buildAndRenderGL(PFGLRendererRef renderer, PFBuildOptionsRef options) {
		try {
			MH_PFSceneProxyBuildAndRenderGL.invokeExact(this.ptr(), renderer.ptr(), options.ptr());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}
}
